"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HashTableUtils = void 0;
class HashTableUtils {
    static valueToString(value) {
        if (value === null || value === undefined)
            return 'null';
        let stringKey;
        switch (typeof value) {
            case 'number':
                stringKey = value.toString();
                break;
            case 'object':
                if (typeof value.toString === 'function') {
                    stringKey = value.toString();
                }
                else {
                    stringKey = JSON.stringify(value);
                }
                break;
            case 'string':
                stringKey = value;
                break;
            case 'function':
                stringKey = value.toString();
                break;
            case 'symbol':
                stringKey = value.toString();
                break;
            default:
                stringKey = String(value);
        }
        return stringKey;
    }
    // Thomas Wang, Integer Hash Functions.
    static wangHash32(key) {
        key = key >>> 0;
        key = ~key + (key << 15);
        key = key ^ (key >>> 12);
        key = key + (key << 2);
        key = key ^ (key >>> 4);
        // Ensure multiplication wraps to 32 bits
        key = ((key << 11) + (key << 3) + key) >>> 0;
        key = key ^ (key >>> 16);
        return key >>> 0;
    }
    /*
     * DJB2a (variant using xor rather than +) hash algorithm.
     * See: http://www.cse.yorku.ca/~oz/hash.html
     */
    static hashFunction(str) {
        let hash = 5381;
        for (let i = 0; i < str.length; i++) {
            hash = ((hash << 5) + hash) ^ str.charCodeAt(i);
        }
        // Convert the hash to an unsigned 32-bit integer to match C's unsigned long.
        return hash >>> 0;
    }
    static hash(key, capacity) {
        if (key && typeof key.hashCode === 'function') {
            return key.hashCode();
        }
        if (typeof key === 'number' && Number.isSafeInteger(key)) {
            return HashTableUtils.wangHash32(key) % capacity;
        }
        const stringKey = this.valueToString(key);
        return this.hashFunction(stringKey) % capacity;
    }
    static keysEqual(key1, key2) {
        if (key1 && key2 && typeof key1.equals === 'function') {
            return key1.equals(key2);
        }
        if (key1 === key2)
            return true;
        if (typeof key1 === 'number' && typeof key2 === 'number' && isNaN(key1) && isNaN(key2))
            return true;
        if (typeof key1 !== 'object' && typeof key2 !== 'object')
            return key1 === key2;
        if (key1 instanceof Date && key2 instanceof Date) {
            // Handle invalid dates
            if (isNaN(key1.getTime()) && isNaN(key2.getTime()))
                return true;
            return key1.getTime() === key2.getTime();
        }
        if (key1 instanceof RegExp && key2 instanceof RegExp) {
            return key1.toString() === key2.toString();
        }
        if (Array.isArray(key1) && Array.isArray(key2)) {
            return key1.length === key2.length &&
                key1.every((val, idx) => HashTableUtils.keysEqual(val, key2[idx]));
        }
        if (key1 === null && key2 === null)
            return true;
        if (key1 === undefined && key2 === undefined)
            return true;
        if (key1 === null || key1 === undefined || key2 === null || key2 === undefined)
            return false;
        if (typeof key1 === 'object' && typeof key2 === 'object') {
            const keys1 = Object.keys(key1);
            const keys2 = Object.keys(key2);
            return keys1.length === keys2.length &&
                keys1.every(k => k in key2 && HashTableUtils.keysEqual(key1[k], key2[k]));
        }
        return false;
    }
}
exports.HashTableUtils = HashTableUtils;
//# sourceMappingURL=hash-table-utils.js.map