"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HashTable = void 0;
const hash_table_utils_1 = require("./hash-table-utils");
class HashNode {
    key;
    value;
    next;
    constructor(key, value) {
        this.key = key;
        this.value = value;
        this.next = null;
    }
}
class HashTable {
    table;
    count;
    capacity;
    constructor(capacity = 4096) {
        // Handle negative or zero capacity by using default capacity
        this.capacity = capacity <= 0 ? 4096 : capacity;
        this.table = new Array(this.capacity).fill(null);
        this.count = 0;
    }
    insert(key, value) {
        const index = hash_table_utils_1.HashTableUtils.hash(key, this.capacity);
        // Handle empty bucket case.
        if (!this.table[index]) {
            this.table[index] = new HashNode(key, value);
            this.count++;
            return;
        }
        // Check first node for key match. If it matches, update the value.
        if (hash_table_utils_1.HashTableUtils.keysEqual(this.table[index].key, key)) {
            this.table[index].value = value;
            return;
        }
        // Traverse chain to find key or last node. If it matches, update the value.
        let current = this.table[index];
        while (current.next) {
            if (hash_table_utils_1.HashTableUtils.keysEqual(current.next.key, key)) {
                current.next.value = value;
                return;
            }
            current = current.next;
        }
        // Key not found, append new node.
        current.next = new HashNode(key, value);
        this.count++;
    }
    get(key) {
        const index = hash_table_utils_1.HashTableUtils.hash(key, this.capacity);
        let current = this.table[index];
        while (current) {
            if (hash_table_utils_1.HashTableUtils.keysEqual(current.key, key)) {
                return current.value;
            }
            current = current.next;
        }
        return undefined;
    }
    remove(key) {
        const index = hash_table_utils_1.HashTableUtils.hash(key, this.capacity);
        let current = this.table[index];
        let prev = null;
        while (current) {
            if (hash_table_utils_1.HashTableUtils.keysEqual(current.key, key)) {
                if (prev) {
                    prev.next = current.next;
                }
                else {
                    this.table[index] = current.next;
                }
                this.count--;
                return true;
            }
            prev = current;
            current = current.next;
        }
        return false;
    }
    size() {
        return this.count;
    }
    isEmpty() {
        return this.count === 0;
    }
    clear() {
        this.table = new Array(this.capacity).fill(null);
        this.count = 0;
    }
}
exports.HashTable = HashTable;
//# sourceMappingURL=hash-table.js.map